#!/usr/local/bin/perl -w

=head1 NAME

cvs.t - testing of vcp cvs i/o

=cut

use strict ;

use Carp ;
use Cwd ;
use File::Spec ;
use Test ;
use VCP::TestUtils ;

my @vcp = vcp_cmd ;

my $t = -d 't' ? 't/' : '' ;

my $module = 'foo' ;  ## Must match the rev_root in the testrevml files

my @revml_out_spec = ( "revml:", "--sort-by=name,rev_id" ) ;

my $max_change_id ;

my $p4_spec_base ;
my $p4d_borken = $ENV{P4DBORKEN} || p4d_borken ;

sub check {
   goto &die if $ENV{FATALTEST} && ! $_[0];
}

my ( $infile_0, $cvsroot_0, $infile_1, $cvsroot_1 ) =
   make_cvsroots( $module, $t );

my $cvs_spec_0        = "cvs:$cvsroot_0:$module/" ;
my $cvs_spec_1        = "cvs:$cvsroot_1:$module/" ;

my @tests = (
sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;

   $ENV{P4USER}   = "foobar_user" ;
   $ENV{P4PORT}   = "foobar_port" ;
   $ENV{P4CLIENT} = "foobar_client" ;
   $ENV{P4PASSWD} = "foobar_passwd" ;

   my $p4_options = launch_p4d "cvs_" ;
   $p4_spec_base = "p4:$p4_options->{user}:\@$p4_options->{port}:" ;
   check ok 1 ;
},

##
## cvs->p4->revml
##
sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;

   my $cvs_spec = $cvs_spec_0;
   my $p4_spec = "$p4_spec_base//depot" ;

   my $vcp_spec = <<TOHERE;
Source:      $cvs_spec_0
Destination: $p4_spec_base//depot
Map:
	(...)<>         main/\$1  # Match files on main branch (only)
        (...)<(*)>      \$2/\$1    # Match files on branches
TOHERE

   eval {
      my $out ;
{ local $ENV{VCPDEBUG}=".*";
      run [ @vcp, "vcp:-" ], \$vcp_spec;
}
      run [ @vcp, "$p4_spec/...", @revml_out_spec ], \undef, \$out;

      my $infile  = $infile_0;
$infile = "t/test-p4-in-0.revml";
      my $in = slurp $infile ;

      s_content  qw( rep_desc time user_id rep_type ), \$in, \$out ;
      s_content  qw( rev_root                       ), \$in, "depot" ;
#      rm_elts    qw( cvs_info mod_time change_id    ), \$in        ;
#      rm_elts    qw( p4_info                        ),       \$out ;
      rm_elts    qw( p4_info                        ), \$in, \$out ;
      rm_elts    qw( label ), qr/r_\w+|ch_\w+/,              \$out ;

$in =~ s{(id="|id>)/ignored}{$1//depot}g;
assert_eq $infile, $in, $out ;

      $out =~ s{<rev_id>}{<rev_id>1.}g ;
      $out =~ s{<base_rev_id>}{<base_rev_id>1.}g ;

      $out =~ s{^.*<change_id>(.*?)</change_id>.*(\r\n|\n\r|\n)}{
	 $max_change_id = $1
	    if ! defined $max_change_id || $1 > $max_change_id ;
	 "" ;
      }gem ;

      # The sort order is different because the name changed, and we want
      # to tweak the name differently in the section to be moved.

      $in =~ s{
         (^\s*<rev\s+id="/ignored/branched.1\.1\.2
            .*?</rev>\n
            .*?</rev>\n
      )}{}smx or die "Can't munge branch-1 out of \$in";

      my $branch_1 = $1;

      $in =~ s{(id="|id>)/ignored}{$1//depot/main}g;
      $in =~ s{(<name[^>]*>)}{$1main/}g;

      $branch_1 =~ s{(id="|id>)/ignored}{$1//depot/branch-1}g;
      $branch_1 =~ s{(<name[^>]*>)}{$1branch-1/}g;
      $branch_1 =~ s{((id="|id>)[^>]*)#1\.1\.2\.}{$1#1.}g;

      $in =~ s{(</branches>\n)}{$1$branch_1};
      
      $out =~ s{((id="|id>)[^>]*)#}{$1#1.}g;

      assert_eq $infile, $in, $out ;
   } ;
   check ok $@ || '', '', 'diff' ;
},

sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;
   check ok $max_change_id, 3, "Max change_id in cvs->p4 transfer" ;
},

##
## cvs->p4->revml, re-rooting a dir tree
##
sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;

   ## Hide global $cvs_spec for the nonce
   my $cvs_spec = "$cvs_spec_0/a/deeply/..." ;

   my $p4_spec = "$p4_spec_base//depot/new/..." ;
   eval {
      my $out ;
      run [ @vcp, $cvs_spec, $p4_spec ], \undef;
      run [ @vcp, $p4_spec, @revml_out_spec ], \undef, \$out;

      my $infile  = $t . $infile_0;
      my $in = slurp $infile ;

      s_content  qw( rep_desc time user_id rep_type  ), \$in, \$out ;
      s_content  qw( rev_root                        ), \$in, "depot/new" ;
      rm_elts    qw( cvs_info mod_time change_id     ), \$in ;
      rm_elts    qw( p4_info                         ),       \$out ;
      rm_elts    qw( label ),  qr/r_\w+|ch_\w+/,              \$out ;


      $out =~ s{<rev_id>}{<rev_id>1.}g ;
      $out =~ s{<base_rev_id>}{<base_rev_id>1.}g ;

      ## Strip out all files from $in that shouldn't be there
      rm_elts    qw( rev ), qr{(?:(?!a/deeply).)*?}s, \$in ;

      ## Adjust the $in paths to look like the result paths.  $in is
      ## now the "expected" output.
      $in =~ s{<name>a/deeply/}{<name>}g ;

      $out =~ s{^.*<change_id>(.*?)</change_id>.*(\r\n|\n\r|\n)}{
	 $max_change_id = $1
	    if ! defined $max_change_id || $1 > $max_change_id ;
	 "" ;
      }gem ;

      $in =~ s{(id="|id>)/ignored/a/deeply}{$1//depot/new}g;
      $out =~ s{((id="|id>)[^>]*)#}{$1#1.}g;

      assert_eq $infile, $in, $out ;
   } ;
   check ok $@ || '', '', 'diff' ;
},

sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;
   check ok $max_change_id, 6, "Max change_id in cvs->p4 transfer" ;
},

##
## Incremental cvs->p4->revml update
##
sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;

   my $cvs_spec = $cvs_spec_1;
   my $p4_spec = "$p4_spec_base//depot" ;

   eval {
      my $out ;
      my $first_change = $max_change_id + 1 ;
      run [ @vcp, $cvs_spec, "-r", "ch_4:", "$p4_spec/..." ], \undef;
      run [ @vcp, "$p4_spec/...\@$first_change,#head", @revml_out_spec ],
         \undef, \$out;
      my $infile  = $infile_1;
      my $in = slurp $infile ;

      s_content  qw( rep_desc time user_id rep_type ), \$in, \$out ;
      s_content  qw( rev_root                       ), \$in, "depot" ;
      rm_elts    qw( cvs_info mod_time change_id    ), \$in        ;
      rm_elts    qw( p4_info                        ),       \$out ;
      rm_elts    qw( label ), qr/r_\w+|ch_\w+/,              \$out ;

      $out =~ s{<rev_id>}{<rev_id>1.}g ;
      $out =~ s{<base_rev_id>}{<base_rev_id>1.}g ;

      $out =~ s{^.*<change_id>(.*?)</change_id>.*(\r\n|\n\r|\n)}{
	 $max_change_id = $1
	    if ! defined $max_change_id || $1 > $max_change_id ;
	 "" ;
      }gem ;

      assert_eq $infile, $in, $out ;
   } ;
   check ok $@ || '', '', 'diff' ;
},

sub {
   return skip $p4d_borken, 1, 1, $p4d_borken if $p4d_borken ;
   check ok $max_change_id, 9, "Max change_id in cvs->p4 transfer" ;
},

) ;

plan tests => scalar( @tests ) ;

my $why_skip ;

$why_skip .= cvs_borken ;
$why_skip ? skip( $why_skip, 0 ) : $_->() for @tests ;
